package com.humantask;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.drools.runtime.process.WorkItemManager;
import org.jbpm.process.audit.JPAWorkingMemoryDbLogger;
import org.jbpm.process.workitem.wsht.BlockingAddTaskResponseHandler;
import org.jbpm.process.workitem.wsht.CommandBasedWSHumanTaskHandler;
import org.jbpm.process.workitem.wsht.HornetQHTWorkItemHandler;
import org.jbpm.process.workitem.wsht.LocalHTWorkItemHandler;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.local.LocalTaskService;
import org.jbpm.task.utils.OnErrorAction;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class SimpleHumanTask {

	/**
	 * @param args
	 * @throws Exception
	 */
	@SuppressWarnings("deprecation")
	public static void main(String[] args) throws InvocationTargetException,
			Exception {
		// TODO Auto-generated method stub

		System.out.println("== Knowledge Base Called ==");

		bitronix.tm.BitronixTransactionManager transactionManager = TransactionManagerServices
				.getTransactionManager();
		transactionManager.begin();
		KnowledgeBase kbase = readKnowledgeBase();

		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("org.jbpm.task");

		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
		env.set(EnvironmentName.TRANSACTION_MANAGER,transactionManager);

		Properties properties = new Properties();

		properties
				.put("drools.processInstanceManagerFactory",
						"org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
		properties.put("drools.processSignalManagerFactory",
				"org.jbpm.persistence.processinstance.JPASignalManagerFactory");

		 KnowledgeSessionConfiguration config = KnowledgeBaseFactory
		 .newKnowledgeSessionConfiguration(properties);

		System.out.println("Session ID 1:->"
				+ kbase.newStatefulKnowledgeSession().getId());

		try {
			
			StatefulKnowledgeSession ksession = JPAKnowledgeService
					.newStatefulKnowledgeSession(kbase, config, env);
			JPAWorkingMemoryDbLogger logger = new JPAWorkingMemoryDbLogger(
					ksession);

					
			 CommandBasedWSHumanTaskHandler taskHandler = new CommandBasedWSHumanTaskHandler(ksession);
			 ksession.getWorkItemManager().registerWorkItemHandler("Human Task",taskHandler);
			 taskHandler.connect();
					
			ProcessInstance process = ksession.createProcessInstance(
					"com.sample.humanprocess", null);
			ksession.insert(process);
			System.out.println("Process Id :->" + process.getId());
			ksession.startProcessInstance(process.getId());

			System.out.println("Human Task Started....");
			System.out.println("Process State:->" + process.getState());
		} catch (Exception e) {
			System.out.println("\n === Message Is ===>" + e.getMessage());
			System.out.println("\n === Stack Trace is ===");
			e.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		@SuppressWarnings("static-access")
		KnowledgeBuilder kbuilder = new KnowledgeBuilderFactory()
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("humanttask.bpmn"),
				ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}

}
